#################################################################################
# Created       : Martin Foster
# Modified      : 14-Dec-2006
#################################################################################
#
# Who - Modules part of Ethereal Realms designed to determine who is on 
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3-3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

package Ethereal::Geo;						# Package name

use Math::Trig;							# Additional functions
use Socket;							# Socket handling library
use strict;							# Anal retentive check for variables


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data Members
	my $class    = shift;					# Self reference
	my $database = shift;					# Database handle

	my $ip2;						# IP2Location.com database
	my $mmd;						# Maxmind.com database
	my $key;						# Current implementation
	
	my %geo;						# List of handlers to use
	

	#####################
	# Program Area

	# Create handle of possibilities
	# Arrange in order of preference
	%geo = (
	  1 => "use Geo::IP\; \$mmd = Geo::IP->open(\'$database->{SYS}{SetGeoMaxMind}\', GEOIP_STANDARD)\;",
	  2 => "use Geo::IP2Location\; \$ip2 = Geo::IP2Location->open(\'$database->{SYS}{SetGeoIP2Location}\')\;"
	 );

	# Determine source
	foreach $key (sort(keys(%geo)))
	{
		# Evaluate the above statement and work accordingly
		eval "$geo{$key}";

		# All that is necessary
		last unless ($@); 
	}

	
	# Setup hash
	my $self = { 
		IP2  => $ip2,					# Stored handle
		MMD  => $mmd					# Stored handle
	};

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


sub numeric
{
	$a <=> $b;
}

#####################
# Geo Distance
#
# Uses a formula from the 'Aviation Formulary V1.42' site by 'Ed Williams' 
# to get extremes for North-East-South-West.   For more reference please
# visit: http://williams.best.vwh.net/avform.htm

sub Distance
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $dis      = shift;					# Distance ranged
	my $olat     = shift;					# Original latitude
	my $olon     = shift;					# Original longitude

	my $pi       = atan2(1,1) * 4;				# Approximate value of PI
	my $tc;							# Degree handling

	my $lat;						# Temporary work lat
	my $lon;						# Temporary work long
	
	my @cardinal = qw(0 90 180 270);			# Cardinal points to calculate
	my @lats;						# Array of lats
	my @lons;						# Array of longs

	
	#####################
	# Data members
	
	# Must convert kilometers to radians
	$dis  = ($pi/(180 * 60 * 1.852)) * $dis;

	# Convert to radians
	$olat = ($pi/180) * $olat;
	$olon = ($pi/180) * $olon;

	# Round and round we go
	foreach $tc (@cardinal)
	{
		# Convert it appropriatly
		$tc = ($pi/180) * $tc;

		# Run the calculations
		# Latitude is the easy part
		$lat  = asin(sin($olat)*cos($dis)+cos($olat)*sin($dis)*cos($tc));
		
		# Determine longitude
		# Endpoint is a poll
		if (cos($olat) == 0)
		{
			# Simple assingment
			$lon=$olon;
		}
		else
		{
			# Do it the old fashioned way
			$lon = ($olon-asin(sin($tc)*sin($dis)/cos($lat))+$pi % 2*$pi)-$pi
		}
		
		# Convert back to degrees
		$lat = (180/$pi) * $lat;
		$lon = (180/$pi) * $lon;

		# Push onto array
		push(@lats,  sprintf("%.3f", $lat));
		push(@lons,  sprintf("%.3f", $lon));
	}

	# Sort
	@lats = sort {$a <=> $b} @lats;
	@lons = sort {$a <=> $b} @lons;

	# Return the entire mess
	return $lats[0], $lons[0], $lats[3], $lons[3];
}


#####################
# Geo Locate
#
# Uses whichever database handle that has successfully been initialized and in
# what order. 

sub Locate
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $address  = shift;					# Pulled address

	my $ip2      = $self->{'IP2'};				# IP2Location instance
	my $mmd      = $self->{'MMD'};				# Maxmind instance

	my $key;						# Testing key
	my $rec;						# Maxmind record
	
	my $lat;						# Latitude
	my $long;						# Longitude
	my $pked;						# Packed IP address
	
	my $city;						# City of origin
	my $region;						# Region of city
	my $country;						# Country of city

	#####################
	# Data members

	# Invalid IP address
	if ($address !~ /\d{1,3}\.\d{1,3}\.\d{1,3}/)
	{
		# Retrieve address
		$pked = gethostbyname($address);

		# Form into human readable
		$address = inet_ntoa($pked);
	}


	# Use of the IP2Location database
	# Check
	if (defined($ip2))
	 {
		# Simple query
		($_, $country, $region, $city, $lat, $long) = $ip2->get_all($address);
	};

	# Maxmind database
	# Check
	if ((defined($mmd))
	 && ((!defined($country)) || ($country =~ /demo version/i)))
 	{
		# Draw a record
		$rec = $mmd->record_by_addr($address);

		# Pull from record
		$country = $rec->country_name;
		$region  = $rec->region;
		$city    = $rec->city;
		$lat     = $rec->latitude;
		$long    = $rec->longitude;
	}
	
	# Make sure its all good
	if ((defined($country))
	 && ($country =~ /demo version/i))
	{
			# Reset accordingly
			$country = undef;
			$region  = undef;
			$city    = undef;
			$lat     = undef;
			$long    = undef;
	}

	# Return everything
	return $country, $region, $city, $lat, $long;
}

1;								# Return true
