#################################################################################
# Created       : Martin Foster
# Modified      : 14-Dec-2006
#################################################################################
#
# Mail - Unified mail sending module for Ethereal Realms
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3-3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

package Ethereal::Mail;						# Package definition

use MIME::Entity;						# Mail MIME handling
use Text::Wrap;							# Text wrapping functionality


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	####################
	# Data members
	my $class    = shift;
	my $database = shift;

	my $self;


	####################
	# Program Area

	# Define internal structure
	%{$self}  = (
		FROM     => "\"$database->{SYS}{SetSenderName}\" <$database->{SYS}{SetSenderAddress}>",
		HANDLE   => $database,				# Database handle
		MAIL     => $database->{'SYS'}{'SetInfoContactAddress'}, # Implied address
		MAILER   => $database->{'SYS'}{'SetSendmail'},	# Location of sendmail	
		SUBJECT  => '',					# Subject of message
		XMIME    => 0,					# MIME handling
		XWRAP    => 0,					# Text message wrapping
		XMAILER  => "$database->{SYS}{SetInfoServer} $database->{SYS}{SetVersion}",
		ATTACH   => [],					# Attachments
		ATTACHED => 0,					# Number of attachments
		BCC      => [],					# Blind Carbon Copy
		CC       => [],					# Carbon Copy
		TARGETS  => 0,					# Total targets to be mailed
		TO       => [],					# Recipient
		XTO      => [],					# Internal to field
		DRAFT    => [],					# Draft message
		SEARCH   => {}					# Search and replace
	);

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}

#####################
# Add Field
#
# This will simply append the appropriate field to the proper array reference
# created when the object was.   This will reduce the complexity inherent to
# the use of certain modules and clean up the mail sending.

sub Add
{

	####################
	# Data members
	my $self     = shift;					# Reference to self
	my $field    = shift;					# Feild to append to
	my $value;						# Specific value


	####################
	# Program Area

	# Find all remaining entries
	while ($value = shift)
	{
		# Push onto array reference
		push(@{$self->{$field}}, $value); 

		# Seperate address from prettiness
		$value = ($value =~ /^.*\<(.+)\>$/) ? $1 : $value;

		# Push onto Internal Address
		push (@{$self->{'XTO'}}, $value);

		# Increment to targets
		$self->{'TARGETS'}++;
	}
}

# Alias for Blind Carbon Copy
sub AddBCC
{ Add(shift, 'BCC', @_); }

# Alias for Carbon Copy
sub AddCC
{ Add(shift, 'CC', @_); }

# Alias for recipient
sub AddTo
{ Add(shift, 'TO', @_); }


####################
# Alias for Attachments
#
# Specifically applies to attachments and make the interface cleaner
# as to not force the user to provide an actual hashed value.

sub AddAttach
{
	####################
	# Data members
	my $self     = shift;					# Reference to self
	my $value;						# Specific value

	my %hash;						# Stored hash


	####################
	# Program Area

	# Find all remaining entries
	while ($value = shift)
	{
		# Assign to hash
		$hash{$value} = shift;

		# Increment to attached
		$self->{'ATTACHED'}++;
	}


	# Push hashref
	push(@{$self->{'ATTACH'}}, \%hash);
}


#####################
# Add From
#
# This will allow someone to change where a message is coming
# from for use in automated remailers.

sub AddFrom
{

	####################
	# Data members
	my $self     = shift;					# Reference to self
	my $field    = shift;					# Feild to append to


	####################
	# Program Area

	# Simply set
	$self->{'FROM'} = $field;

	# Return
	return 1;
}


#####################
# Set Search
#
# Sets the search criteria used for the replacement of certain macros
# used within the main body of the Email.   This will only be used for
# the main body and not for attachments.

sub SetSearch
{

	####################
	# Data members
	my $self     = shift;					# Reference to self
	my $key;						# Specific key


	####################
	# Program area

	# Loop and assign
	while ($key = shift)
	{
		# Assign to hash within harhref
		$self->{'SEARCH'}->{$key} = shift;
	}
}


#####################
# Set Subject
#
# Will set the subject of the Email hiding the complexity that may be
# inherent in the Emails contruction

sub SetSubject
{

	####################
	# Data members
	my $self     = shift;					# Reference to self
	my $subj     = shift;					# Subject line


	####################
	# Program area

	# Simple assignment
	$self->{'SUBJECT'} = $subj; 
}


#####################
# Toggle Wrap
#
# Exists in order toggle the wrapping behaviour shown when creating text
# messages.    The default for these is off, toggling will switch it on or
# off as needed.

sub Toggle
{

	####################
	# Data members
	my $self     = shift;					# Reference to self
	my $toggle   = shift;					# Toggle Value


	####################
	# Program area

	# Dirt simple check
	# Text wrap
	$self->{$toggle} = ($self->{$toggle} > 0) ? 0 : 1; 
}

# Alias for Wrapping of Text
sub ToggleWrap
{ Toggle(shift, 'XWRAP'); }


#####################
# Write
#
# Allows us to employ formatting without actually making use of the write
# statements which are not supported or seem not to be.

sub Write
{
	####################
	# Data members
	my $self     = shift;					# Reference to self
	my $format   = shift;					# Format lines


	####################
	# Program area

	# Ensure clearance
	$^A = '';

	# Format line
	formline($format, @_);

	# Append lines
	push(@{$self->{DRAFT}}, $^A);
}


#####################
# TIEHANDLE
#
# Needed in order to create the effect of a filehandle.  This in itself
# will allow for control that is needed and would be lost without it.
# In this case it will make use of the existing new structure as to 
# allow for multiple uses.

sub TIEHANDLE
{ return new(shift, shift); }


#####################
# PRINT
#
# Allows this class to understand how to work with the PRINT statement

sub PRINT
{
	####################
	# Data members
	my $self     = shift;					# Reference to self


	####################
	# Program area

	# Append lines
	push(@{$self->{DRAFT}}, @_);
}


#####################
# PRINTF
#
# Allows this class to understand how to work with the PRINTF statement

sub PRINTF
{
	####################
	# Data members
	my $self     = shift;					# Reference to self
	my $template = shift;					# Formatting to use


	####################
	# Program area

	# Append lines
	push(@{$self->{DRAFT}}, sprintf($template, @_));
}


#####################
# CLOSE
#
# Cleanup and sending off of the actual Email.   Behaviour will vary
# based on MIME setting.

sub CLOSE
{
	####################
	# Data members
	my $self     = shift;					# Reference to self

	my $database = $self->{'HANDLE'};			# Database connection

	my $body;						# Mail body
	my $top;						# MIME handle

	my $address;						# Address field
	my $mime;						# MIME attachment
	my $field;						# Singular field
	my $server;						# Server addresses
	my $value;						# Replaced value

	my $mailer;						# Mailer preferences
	my $target;						# User fetching mail
	

	####################
	# Program area

	# Body mangling
	# Build body
	$body = join('', @{$self->{'DRAFT'}});

	# Macro replacement
	foreach $field (keys(%{$self->{'SEARCH'}}))
	{
		# Definition
		$value = $self->{'SEARCH'}->{$field};
		$value = (defined($value)) ? $value : '';
		
		# Remove key
		delete $self->{'SEARCH'}->{$field};

		# Search and replace
		$body =~ s/$field/$value/gs;
	}	



	# Verify need for internal mail
	if (($self->{'ATTACHED'} < 1)
	 and ($self->{'TARGETS'} < 2))
	{
		# Mailer settings and login name
		($mailer, $target) = $database->DataGetPuppeteerMailer($self->{'XTO'}[0]);
	}
	
	# Internal Mail behaviour 	
	if ((defined($mailer))
	  and (defined($target))
          and ($mailer eq 'int'))
	{
		# Create internal mail entry
		$database->Write("INSERT INTO Mail
			(MailRcptLogin,
			 MailSubject,
			 MailContents) VALUES (?,?,?)",

		  $target,
		  $self->{'SUBJECT'},
		  $body
	  	);
	}
	
	# External Mail behaviour
	else
	{
		# Text wrapping
		if ($self->{'XWRAP'} > 0)
		{
			# Set wrapping width
			$Text::Wrap::columns = 72;		

			# Build body
			$body = wrap('', '', $body);
		}


		# Mail creation
		# Initialize header
 		$top = MIME::Entity->build(
			'Type'     => "multipart/mixed",
			'From'     => $self->{'FROM'},
			'To'       => $self->{'TO'},
			'CC'       => $self->{'CC'},
			'BCC'      => $self->{'BCC'},
			'Subject'  => $self->{'SUBJECT'},
			'X-Mailer' => $self->{'XMAILER'}
		 );


		# Text body
		$top->attach(Data=> $body);


		# Additional attachment
		while ($mime = shift(@{$self->{'ATTACH'}}))
		{
			# Append attachments
			$top->attach(%{$mime}) if ref($mime);
		}


		# Send out mail	
		# Setup sendmail
		open(SENDMAIL, "|$self->{MAILER}") or die "Cannot open $self->{MAILER} $!";

		# Print message
		print SENDMAIL $top->stringify();

		# Close and send
		close(SENDMAIL);
	}
}

1;								# Return True
